"use strict";
// DEVELOPMENT NOTE:
// Please ensure that any modifications made to this file are also applied to the following locations:
// 1) src/repositories/default-transaction-soft-crud.repository.base.ts
Object.defineProperty(exports, "__esModule", { value: true });
exports.SoftCrudRepository = void 0;
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const lodash_1 = require("lodash");
const soft_filter_builder_1 = require("./../utils/soft-filter-builder");
class SoftCrudRepository extends repository_1.DefaultCrudRepository {
    constructor(entityClass, dataSource, getCurrentUser) {
        super(entityClass, dataSource);
        this.getCurrentUser = getCurrentUser;
    }
    find(filter, options) {
        const modifiedFilter = new soft_filter_builder_1.SoftFilterBuilder(filter)
            .imposeCondition({
            deleted: false,
        })
            .build();
        return super.find(modifiedFilter, options);
    }
    findAll(filter, options) {
        return super.find(filter, options);
    }
    findOne(filter, options) {
        const modifiedFilter = new soft_filter_builder_1.SoftFilterBuilder(filter)
            .imposeCondition({
            deleted: false,
        })
            .build();
        return super.findOne(modifiedFilter, options);
    }
    // findOne() including soft deleted entry
    findOneIncludeSoftDelete(filter, options) {
        return super.findOne(filter, options);
    }
    async findById(id, filter, options) {
        const originalFilter = filter !== null && filter !== void 0 ? filter : {};
        const idProp = this.entityClass.getIdProperties()[0];
        const modifiedFilter = new soft_filter_builder_1.SoftFilterBuilder((0, lodash_1.cloneDeep)(originalFilter))
            .imposeCondition({
            deleted: false,
            [idProp]: id,
        })
            .limit(1)
            .build();
        const entity = await super.find(modifiedFilter, options);
        if (entity && entity.length > 0) {
            return entity[0];
        }
        else {
            throw new rest_1.HttpErrors.NotFound("EntityNotFound" /* ErrorKeys.EntityNotFound */);
        }
    }
    // findById (including soft deleted record)
    async findByIdIncludeSoftDelete(id, filter, options) {
        //As parent method findById have filter: FilterExcludingWhere<E>
        //so we need add check here.
        const entity = await super.findOne(filter, options);
        if (entity) {
            // Now call super
            return super.findById(id, filter, options);
        }
        else {
            throw new rest_1.HttpErrors.NotFound("EntityNotFound" /* ErrorKeys.EntityNotFound */);
        }
    }
    updateAll(data, where, options) {
        const filter = new soft_filter_builder_1.SoftFilterBuilder({ where })
            .imposeCondition({
            deleted: false,
        })
            .build();
        return super.updateAll(data, filter.where, options);
    }
    count(where, options) {
        const filter = new soft_filter_builder_1.SoftFilterBuilder({ where })
            .imposeCondition({
            deleted: false,
        })
            .build();
        return super.count(filter.where, options);
    }
    countAll(where, options) {
        return super.count(where, options);
    }
    // soft delete
    async delete(entity, options) {
        return this.deleteById(entity.getId(), options);
    }
    async deleteAll(where, options) {
        const deletedBy = await this.getUserId(this.getCurrentUser);
        const dataToUpdate = {
            deleted: true,
            deletedOn: new Date(),
            deletedBy,
        };
        return super.updateAll(dataToUpdate, where, options);
    }
    // soft delete by id
    async deleteById(id, options) {
        const deletedBy = await this.getUserId(this.getCurrentUser);
        return super.updateById(id, {
            deleted: true,
            deletedOn: new Date(),
            deletedBy,
        }, options);
    }
    /**
     * Method to perform hard delete of entries. Take caution.
     * @param entity
     * @param options
     */
    deleteHard(entity, options) {
        // Do hard delete
        return super.deleteById(entity.getId(), options);
    }
    /**
     * Method to perform hard delete of entries. Take caution.
     * @param entity
     * @param options
     */
    deleteAllHard(where, options) {
        // Do hard delete
        return super.deleteAll(where, options);
    }
    /**
     * Method to perform hard delete of entries. Take caution.
     * @param entity
     * @param options
     */
    deleteByIdHard(id, options) {
        // Do hard delete
        return super.deleteById(id, options);
    }
    async getUserId(options) {
        var _a, _b;
        if (!this.getCurrentUser) {
            return undefined;
        }
        let currentUser = await this.getCurrentUser();
        currentUser = currentUser !== null && currentUser !== void 0 ? currentUser : options === null || options === void 0 ? void 0 : options.currentUser;
        if (!currentUser) {
            return undefined;
        }
        const userId = (_b = (_a = currentUser.getIdentifier) === null || _a === void 0 ? void 0 : _a.call(currentUser)) !== null && _b !== void 0 ? _b : currentUser.id;
        return userId === null || userId === void 0 ? void 0 : userId.toString();
    }
}
exports.SoftCrudRepository = SoftCrudRepository;
//# sourceMappingURL=soft-crud.repository.base.js.map